<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
</head>

<body>
<p>Hi,</p>
<p>A new contact request has been submitted by Mr/Mrs {{.Name}}.</p>
<ul>
  <li>Name: {{.Name}}</li>
  <li>Email: {{.Email}}</li>
  <li>Organization: {{.Organization}}</li>
  <li>Subject: {{.Subject}}</li>
  <li>Message: {{.Message}}</li>
</ul>
--
<p>This is an automatic email from Krossboard's website.</p>

</body>

</html>